import os
import random
import time
from typing import Union, List

from nonebot import require, logger
from nonebot.adapters.onebot.v11 import Bot, Message, MessageSegment, GroupMessageEvent, PrivateMessageEvent
from nonebot.exception import ActionFailed
from nonebot.matcher import Matcher
from nonebot.params import CommandArg

try:
    scheduler = require("nonebot_plugin_apscheduler").scheduler
except Exception:
    scheduler = None
    logger.warning("未安装定时插件依赖")

from .Config import plugin_config

from .Ess import ess

class Check:
    def __init__(self) -> None:
        self.cd = plugin_config.le_cd
        self.max_count = plugin_config.le_max
        self.user_cd = {}
        self.user_count = {}
        def reset_user_count():
            self.user_count = {}

        try:
            scheduler.add_job(reset_user_count, "cron", hour="0", id="clear_le_max_count")
        except ActionFailed as e:
            logger.warning(f"定时任务添加失败，{repr(e)}")
        
    def check_cd(self, event: Union[GroupMessageEvent, PrivateMessageEvent]) -> bool:
        if self.cd == 0:
            return False
        
        user_id = event.user_id
        current_time = int(time.time())
        
        if str(user_id) not in list(self.user_cd.keys()):
            self.user_cd[f"{user_id}"] = current_time
            return False
        
        delta_time = current_time - self.user_cd[f'{user_id}']
        
        if delta_time >= self.cd:
            self.user_cd[f'{user_id}'] = current_time
            return False
        else:
            return True
    
    def check_max(self, event: Union[GroupMessageEvent, PrivateMessageEvent]) -> bool:
        if self.max_count == 0:
            return False
        
        user_id = event.user_id
    
        if str(user_id) not in list(self.user_count.keys()):
            self.user_count[f"{user_id}"] = 0

        if self.user_count[f"{user_id}"] < self.max_count:
            self.user_count[f"{user_id}"] += 1
            return False
        else:
            return True
        
    def check(self, event: Union[GroupMessageEvent, PrivateMessageEvent]) -> bool:
        return check.check_cd(event) or check.check_max(event)

class Le:
    def __init__(self) -> None:
        pass

    async def lulu(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent]) -> None:
        if not ess.check(event, 'Le'):
            return None
        
        if check.check(event):
            return None
        
        deckList = ["冲刺！","纱季的曲奇","迁怒于人","远距离射击","美妙的叮铃铃","美妙的礼物","坚固的结晶","布丁","残机奖励","大小姐的特权",
            "孤独的暴走车","偷袭","探究心","学生会长的权益","模仿","成功报酬","绅士的决斗","宝物小偷","拦路者","幕后交易","加班","我的野生朋友们","传送操控",
            "燃起来吧！","兔子浮游炮","虹色之环","大麦格农炮","护盾","最终决战","性格反转力场","临阵脱逃","护盾反转","极速超能",
            "部件扩张","瞬间再生","阴谋的资金筹集","自暴自弃的改造","赌命一搏","殊死对决","便携布丁","虚假除械","可爱大对决",
            "糟糕的布丁","米缪冲击锤","危险布丁","存钱罐","袭来","飞一边去","热度300%","突击","深夜的惨剧","互相交换","喷射火焰","空中餐厅「纯洁」",
            "为了玩具店的未来","小鸡小鸡大游行","记忆封印","无情的恶作剧","礼物小偷","通缉令","啵噗化","想要见到你","安可","香蕉奈奈子","啵噗银行",
            "这里那里","大群的海鸥","神圣之夜","弹药用尽","礼物交换","我们是阴谋组","晚餐","超绝认真模式",
            "强制复活","小小的战争","我的朋友呀","被封印的守护者","混成化现象","恐怖的推销","众神的嬉戏","大争夺·圣诞夜","烧毁星球之光","派对时间",
            "加速空域","宁静","劳而无薪","无差别火力支援","宠物零食","家居翻修","甜点的破坏者","宠物零食","反抗",
            "不幸护符","疾风附魔","迷路的孩子","力量的代价","掠夺者啵噗","鲜血渴望","幸运护符","全金属单体结构","幸运7","奈奈子的浮游炮",
            "露露的幸运蛋","露露的幸运蛋","露露的幸运蛋","露露的幸运蛋","露露的幸运蛋","露露的幸运蛋","露露的幸运蛋","露露的幸运蛋","露露的幸运蛋","露露的幸运蛋",
            "露露的幸运蛋","露露的幸运蛋","露露的幸运蛋","露露的幸运蛋","露露的幸运蛋","露露的幸运蛋","露露的幸运蛋"]
        
        image_path = os.path.join(os.path.dirname(__file__), 'resources', 'lulu')
        
        effect = random.randint(0, 2)
        if random.random() <= 0.99:
            if effect == 0:
                dice = random.randint(1, 6)
                await matcher.finish(MessageSegment.image(os.path.join(image_path, 'le0.png')) + f'露露的幸运蛋结果：\n投掷骰子，获得 {dice*20} Stars')
            elif effect == 1:
                deck = []
                for i in range(0, 5):
                    deck.append('「' + random.choice(deckList) + '」')
                await matcher.finish(MessageSegment.image(os.path.join(image_path, 'le1.png')) + f'露露的幸运蛋结果：\n抽取 5 张卡\n{", ".join(deck)}')
            else:
                await matcher.finish(MessageSegment.image(os.path.join(image_path, 'le2.png')) + '露露的幸运蛋结果：\n恢复所有HP，获得永久ATK、DEF、EVD+1')
        else:
            if effect == 0:
                await matcher.finish(MessageSegment.image(os.path.join(image_path, 'le0b.png')) + '『脸伸过来！』')
            if effect == 1:
                await matcher.finish(MessageSegment.image(os.path.join(image_path, 'le1b.png')) + '『蓝碟！』')
            else:
                await matcher.finish(MessageSegment.image(os.path.join(image_path, 'le2b.png')) + '『露露的幸运蛋壳！』')

    async def nanako(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent]) -> None:
        if not ess.check(event, 'Le'):
            return None
        
        if check.check(event):
            return None

        points = [0, 0, 0]
        for i in range(0, 7):
            points[random.randint(0, 2)] += 1
        await matcher.finish(f'浮游炮展开结果：\nATK+{points[0]} DEF+{points[1]} EVD+{points[2]}')
    
    async def nico(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        if not ess.check(event, 'Le'):
            return None
        
        if check.check(event):
            return None

        text = arg.extract_plain_text()
        num = min(int(text), 9) if (text := text[:1]).isdigit() and int(text) else 4
    
        mw_list = ["平安的旅途","风信鸡的指引","烹饪时间","极速装置","束发魔女","主角的特权","甜点守护者","愤怒狂暴","湛蓝乌鸦·再临","给你的礼物",
            "反射外装","决杀手术","奇迹红豆冰淇淋","星之再生","金蛋","三角力场","变化无常的风车","亚空间通道","阴谋机器人出动！","恶魔之手",
            "阴谋的间谍行动～预备","坚实魔女","空间的跳跃(标记)","特别舞台","飞艇轰炸","束缚之锁","情报官","魔法狱火","记录重现","荒唐的性能",
            "阴谋的操纵者","兔子模玩店","扩散光子步枪","永恒的观测者","浮游炮展开","融化的记忆","圣诞小姐的工作","海贼在天上飞？","店铺扩张战略","天使之手",
            "超能模式！","永久流放","修罗场模式","玩偶使","白色圣诞大粉碎","赌博！","涡轮满载","×16大火箭","大爆炸铃","脱衣","隐身启动","劲敌","杀戮魔法",
            "神出鬼没","兽之魔女","不动之物体","艾莉的奇迹","爆燃！","自爆","火箭炮","大火箭炮","乔纳桑·速袭","另一个最终兵器","露露的幸运蛋","星球的导火线",
            "才气觉醒","水晶障壁","统治者","急速的亚莉希安罗妮","圣露眼","拜托了厨师长！","升档","炽热的商人之魂","月夜之舞","社交界亮相","黄昏色的梦",
            "小麦格农炮","梦寐以求的世界","星星收集者","甜点天堂","甜点制作者的魔法","甜点成堆大作战","假想越狱","露露是厄运之龙","母亲之力","艾莉的超能奇迹","前主角的光辉时刻"]

        result = []
        for i in range(0, num):
            result.append('「' + random.choice(mw_list) + '」')
        await matcher.finish(f'奇迹漫步结果：\n{", ".join(result)}')

    async def divination(self, bot: Bot, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        if not ess.check(event, 'Le'):
            return None
        
        if check.check(event):
            return None
        
        arg = arg.extract_plain_text()
        now = time.localtime()
        seed = round(abs(hash("Polaris_Light" + str(now.tm_yday)+ str(now.tm_year) + "XYZ") / 3.0 + hash("QWERTY" + arg + str(event.user_id) + "0*8&6" + str(now.tm_mday) + "96485.3329") / 3.0))
        random.seed(seed)

        divination_list: List[str] = ['大吉', '吉', '中吉', '小吉', '半吉', '末吉', '末小吉', '凶', '小凶', '半凶', '末凶', '大凶']
        result = random.choice(divination_list)

        if '女装' in arg:
            result = '必须大吉！建议马上开始'

        for name in bot.config.nickname:
            if name in arg:
                result = '你好像有那个大病'

        user_name = event.sender.card if event.sender.card != '' else event.sender.nickname
        now = time.localtime()
        await matcher.finish(f'今天是{now.tm_year}年{now.tm_mon}月{now.tm_mday}日\n{user_name}所求事项：【{arg}】\n\n结果: 【{result}】')

check = Check()
le = Le()


from nonebot.plugin import PluginMetadata

from .Config import Config

__plugin_meta__ = PluginMetadata(
    name='100orangejuice',
    description='100orangejuice',
    usage= r'See Sorabot Docs.',
    type="application",
    supported_adapters={"~onebot.v11"},
    homepage="https://github.com/FDCraft/nonebot-plugin-orangejuice",
    config=Config,
    extra={
        'author': 'Polaris_Light',
        'version': '0.10.0',
        'priority': 10
    }
)

import re

from nonebot import on_command, on_regex
from nonebot.matcher import Matcher

from .Ess import ess

from .Card import card
from .Deck import deck
from .Emote import emote
from .Le import le
from .Mixer import mixer
from .Stats import stats

@on_command('#help', aliases={'#帮助'}, priority=10, block=True).handle()
async def help(matcher: Matcher):
    help_msg = '''100OrangeJuice
    #ess help 基础管理模块帮助
    #card help 查卡帮助
    #icon help 查图标帮助
    #stat help 统计帮助
    #deck=[code] 卡组查看
    #lulu 幸运蛋
    #mw [num] 奇迹漫步
    #7 浮游炮
    #zb [something] 占卜'''
    await matcher.finish(help_msg)

on_command(
    '#ess',
    aliases={'#Ess', '#Essential', '#essentilasx'},
    priority=10,
    block=True,
    handlers=[ess.ess]
)

on_command(
    '#card',
    priority=10,
    block=True,
    handlers=[card.card]
)

on_command(
    '#icon',
    priority=10,
    block=True,
    handlers=[card.icon]
)

on_command(
    '#stat',
    aliases={'#stats', '#sd'},
    priority=10,
    block=True,
    handlers=[stats.stats]
)

on_command(
    '#m',
    priority=10,
    block=True,
    handlers=[mixer.mixer]
)

on_command(
    '#deck=',
    priority=10,
    block=True,
    handlers=[deck.deck]
)

on_command(
    '#lulu',
    aliases={'#幸运蛋'},
    priority=10,
    block=True,
    handlers=[le.lulu]
)

on_command(
    '#mw',
    aliases={'#奇迹漫步'},
    priority=10,
    block=True,
    handlers=[le.nico]
)

on_command(
    '#7',
    aliases={'#浮游炮'},
    priority=10,
    block=True,
    handlers=[le.nanako]
)

on_command(
    '#zb',
    aliases={'#占卜'},
    priority=10,
    block=True,
    handlers=[le.divination]
)


on_regex(
    r"^\:[a-z0-9_]+\:$",
    flags=re.IGNORECASE,
    handlers=[emote.emote])





import json
import os
import re
from typing import Dict, Union

from nonebot import logger, get_driver

import aiohttp
import aiosqlite

from nonebot import logger
from nonebot.adapters.onebot.v11 import GroupMessageEvent, PrivateMessageEvent, Message, MessageSegment
from nonebot.matcher import Matcher
from nonebot.params import CommandArg

from .Config import plugin_config
from .Ess import ess

steam_id_file_path: str = os.path.join(plugin_config.oj_data_path, 'steam_id.db')

class Stats:
    def __init__(seld) -> None:
        pass
    async def init_database(self) -> 'Stats':
        if not os.path.exists(steam_id_file_path):
            if not os.path.exists(plugin_config.oj_data_path):
                os.makedirs(plugin_config.oj_data_path)
                                
            self.db = await aiosqlite.connect(steam_id_file_path)
            self.cursor = await self.db.cursor()

            await self.cursor.execute("CREATE TABLE IF NOT EXISTS steamInfo(qq VARCHAR(11) PRIMARY KEY,steam64id CHARACTER(17),renderType INTERGER,sp1 TINYINT,\
                                sp1_1 TINYINT,sp1_2 TINYINT,sp1_3 TINYINT,sp1_4 TINYINT,sp1_5 TINYINT,sp1_6 TINYINT,sp1_31 TINYINT,sp1_37 TINYINT)")
            await self.cursor.execute("INSERT INTO steamInfo VALUES (?, ?, 0, 2, 1, 1, 1, 1, 1, 1, 1, 1)", ('995905922', '76561198857827726')) # Developer Sign
            await self.cursor.execute("INSERT INTO steamInfo VALUES (?, ?, 0, 31, 1, 0, 0, 1, 0, 0, 1, 1)", ('535369354', '76561198361135527')) # Developer Sign

            if os.path.exists(os.path.join(plugin_config.oj_data_path, 'steam_id.json')): # Change old json into database
                with open(os.path.join(plugin_config.oj_data_path, 'steam_id.json'), 'r', encoding='utf-8') as f:
                    data: Dict[str, str] = json.load(f)
                    for qq, steam64id in data.items():
                        await self.cursor.execute("INSERT INTO steamInfo VALUES (?, ?, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)", (qq, steam64id))

            await self.db.commit()
            await self.db.close()

        self.db = await aiosqlite.connect(steam_id_file_path)
        self.cursor = await self.db.cursor()

        return self
    
    async def help(self, matcher: Matcher) -> None:
        help_msg: str = '''橙汁个人统计图片生成
    #stat <steam64id> [limit]
    用于直接生成。limit为行数，可以不填，默认为5。
    使用非初始样式时，不会生成PVP各角色出场次数和胜场的行。
    #stat bind <steam64id>
    用于将当前账号绑定到对应steam账户。重复使用会更新绑定。
    #stat unbind
    删除自己的绑定。
    #stat me [limit]
    绑定steam64id后，使用本命令来快速生成自己的资料。
    #stat type <type>
    更改出图类型。'''
        await matcher.finish(help_msg)

    async def bind(self, uid: str, steam64id: str, matcher: Matcher) -> None:
        try:
            await self.cursor.execute('INSERT INTO steamInfo VALUES (?, ?, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0) ON CONFLICT(qq) DO UPDATE SET steam64id = ?', (uid, steam64id, steam64id))
            await self.db.commit()
            await matcher.send('绑定成功~')
            return None
        except Exception as e:
            await matcher.send('诶鸭出错啦~绑定失败~')
            raise e
        
    async def unbind(self, uid: str, matcher: Matcher) -> None:
        try:
            await self.cursor.execute('DELETE FROM steamInfo WHERE qq = ?', (uid,))
            await self.db.commit()
            await matcher.send('解绑成功~')
            return None
        except Exception as e:
            await matcher.send('诶鸭出错啦~解绑失败~')
            raise e
        
    async def type(self, uid: str, type: str, matcher: Matcher):
        try:
            await self.cursor.execute('SELECT * FROM steamInfo WHERE qq = ?', (uid,))
            data = await self.cursor.fetchall()
            if data == [] or data is None:
                await matcher.send('你还没有绑定过steam哦~请使用 #stat bind <steam64id> 来进行绑定。')
                return None        
            
            await self.cursor.execute('UPDATE steamInfo SET renderType = ? WHERE qq = ?', (type, uid))
            await self.db.commit()
            await matcher.send('修改成功~')
        except Exception as e:
            await matcher.send('诶鸭出错啦~')
            raise e
        
    async def pin(self, uid: str, pin: str, matcher: Matcher):
        try:
            await self.cursor.execute('SELECT * FROM steamInfo WHERE qq = ?', (uid,))
            data = await self.cursor.fetchall()
            if data == [] or data is None:
                await matcher.send('你还没有绑定过steam哦~请使用 #stat bind <steam64id> 来进行绑定。')
                return None
            
            if pin == '0':
                await self.cursor.execute('UPDATE steamInfo SET sp1 = ? WHERE qq = ?', (pin, uid))
                await self.db.commit()
                await matcher.send('修改成功~')
            else:
                match pin:
                    case '31':
                        index = 10
                    case '37':
                        index = 11
                    case _:
                        index = int(pin) + 3
                if data[0][index]:
                    await self.cursor.execute('UPDATE steamInfo SET sp1 = ? WHERE qq = ?', (pin, uid))
                    await self.db.commit()
                    await matcher.send('修改成功~')                    
                else:
                    await matcher.send('你无法使用这个 pin。')
        except Exception as e:
            await matcher.send('诶鸭出错啦~')
            raise e
 
    async def send_stats(self, matcher: Matcher, uid: str = None, steam64id: str = None, at: str = None, limit: str = 5) -> None:
        try:
            if uid:
                reply = '唔~你还没有设置Steam个人资料为公开呢~'
                await self.cursor.execute('SELECT steam64id,renderType,sp1 FROM steamInfo WHERE qq = ?', (uid,))
                data = await self.cursor.fetchall()
                if data == [] or data is None:
                    await matcher.send('你还没有绑定过steam哦~请使用 #stat bind <steam64id> 来进行绑定。')
                    return None
                
            if steam64id:
                reply = '唔~Ta还没有设置Steam个人资料为公开呢~'
                await self.cursor.execute('SELECT steam64id,renderType,sp1 FROM steamInfo WHERE steam64id = ?', (steam64id,))
                data = await self.cursor.fetchall()
                if data == [] or data is None:
                    data = ((steam64id, 0, 0),)

            if at:
                uid = re.sub(r'\[at:qq=(.*?)\]', r'\1', at)
                reply = '唔~Ta还没有设置Steam个人资料为公开呢~'
                await self.cursor.execute('SELECT steam64id,renderType,sp1 FROM steamInfo WHERE qq = ?', (uid,))
                data = await self.cursor.fetchall()
                if data == [] or data is None:
                    await matcher.send('Ta还没有绑定过steam哦~')
                    return None

            
            img = f'https://interface.100oj.com/stat/render.php?steamid={data[0][0]}&render={data[0][1]}&sp1={data[0][2]}&limit={limit}'
            
            async with aiohttp.ClientSession() as session:
                req = await session.get(img)
                data = await req.read()
            
            if data == b'':
                await matcher.send(reply)
            else:
                await matcher.send(MessageSegment.image(img))
                
        except Exception as e:
            await matcher.send('诶鸭出错啦~')
            raise e



    async def stats(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        if not ess.check(event, 'Stats'):
            return None

    
        rich_text: str = arg.to_rich_text()

        while '  ' in rich_text:
            rich_text.replace('  ', ' ')

        args = rich_text.split(' ')

        while len(args) < 3:
            args.append('')
        logger.debug(f'参数解析结果：{args}')


        match args[0]:
            case 'help' | '':
                await self.help(matcher)
            case 'bind' | '-b':
                uid = str(event.user_id)
                match args[1]:
                    case '':
                        await matcher.finish('请输入steam64id。')
                    case steam64id if steam64id.isdigit() and steam64id.startswith('7656') and len(steam64id) == 17:
                        steam64id = args[1]
                        await self.bind(uid, steam64id, matcher)
                    case _:
                        await matcher.finish('这不是一个有效的steam64id。它应该是一个以7656开头、总长度为17位的纯数字。\n如果您不知道什么是steam64id，推荐在网络上了解更多的相关知识。')
            case 'unbind' | '-u':
                uid = str(event.user_id)
                await self.unbind(uid, matcher)
            case 'type' | '-t':
                uid = str(event.user_id)
                match args[1]:
                    case '':
                        await matcher.finish('请输入出图类型代码。')
                    case type if int(type) in range(0, 7):
                        type = args[1]
                        await self.type(uid, type, matcher)
                    case _:
                        await matcher.finish('这不是一个有效的出图类型代码。它应该介于0与6之间。')
            case 'pin' | '-p':
                uid = str(event.user_id)
                match args[1]:
                    case '':
                        await matcher.finish('请输入pin代码。')
                    case pin if int(pin) in range(0, 7) or pin == '31' or pin == '37':
                        pin = args[1]
                        await self.pin(uid, pin, matcher)
                    case _:
                        await matcher.finish('这不是一个有效的pin代码。请在 https://interface.100oj.com/stat/pininfo.html 查看所有类型。')
            case 'me':
                uid = str(event.user_id)
                match args[1]:
                    case limit if limit.isdigit():
                        limit: str = str(min(int(args[1]), 10))
                        await self.send_stats(matcher, uid=uid, limit=limit)
                    case _:
                        await self.send_stats(matcher, uid=uid)
            case at if at.startswith('[at'):
                at = args[0]
                match args[1]:
                    case limit if limit.isdigit():
                        limit: str = str(min(int(args[1]), 10))
                        await self.send_stats(matcher, at=at, limit=limit)
                    case _:
                        await self.send_stats(matcher, at=at)
            case steam64id if steam64id.isdigit() and steam64id.startswith('7656') and len(steam64id) == 17:
                steam64id = args[0]
                match args[1]:
                    case limit if limit.isdigit():
                        limit: str = str(min(int(args[1]), 10))
                        await self.send_stats(matcher, steam64id=steam64id, limit=limit)
                    case _:
                        await self.send_stats(matcher, steam64id=steam64id)
            case _:
                await self.help(matcher)

stats: Stats = Stats()
driver = get_driver()

@driver.on_startup
async def init_card() -> None:
    await stats.init_database()



from typing import Optional, List, Literal
from pydantic import BaseModel

try:
    # pydantic v2
    from nonebot import get_plugin_config
except ImportError:
    # pydantic v1
    from nonebot import get_driver

class Config(BaseModel):
    oj_data_path: Optional[str] = 'data/100oj'
    match_score: Optional[int] = 75
    le_cd: Optional[int] = 60
    le_max: Optional[int] = 10

plugin_config = None

try:
    # pydantic v2
    plugin_config = get_plugin_config(Config)
except:
    # pydantic v1
    plugin_config = Config.parse_obj(get_driver().config)


import json
import os
import re
from typing import Dict, Tuple, Union

import aiomysql
from fuzzywuzzy import fuzz

from nonebot import get_driver
from nonebot.adapters.onebot.v11 import Message, MessageSegment, GroupMessageEvent, PrivateMessageEvent 
from nonebot.matcher import Matcher
from nonebot.params import CommandArg

from .Config import plugin_config
from .Ess import ess

BaseClass = Tuple[Tuple[str, str, str, str]]

tables = ['cardDeck', 'cardHyper', 'cardOther', 'unitCharacter', 'unitNPC'] # cardBossSkill is special so not in here.
groups = {'原版': 'ordinary', '合作': 'coop', '赏金': 'bounty', '其他': 'other'}

def B2Q(string: str):
    return string.replace('~', '～').replace('&', '＆').replace('(', '（').replace(')', '）').replace('!', '！')

class Card:
    def __init__(self) -> None:
        pass
    async def get_data(self):
        self.db = await aiomysql.connect(
            host='47.242.108.196',
            port=3306,
            user='guest',
            password='Sumika!System2',
            db='data_oj'
        )

        cursor = await self.db.cursor()

        for table in tables:
            await cursor.execute(f"SELECT i18nkey, name_, nameEn, groupData FROM {table}")
            setattr(self, table, await cursor.fetchall())
        
        self.db.close()
        
        regex_path = os.path.join(os.path.dirname(__file__), 'resources', 'card', 'regex.json')
        self.regex: Dict[str, str] = json.load(open(regex_path, 'r', encoding='utf-8'))
        
        return self

    async def help(self, matcher: Matcher):
        help_msg: str = '''橙汁查卡
    #card <name> [group] [table]
    查卡
    #icon <name> [group] [table]
    查图标
    name为中文卡名或者使用下划线的英文卡名
    group为 原版/合作/赏金/其他 或不填
    table为卡表，具体见数据库'''
        await matcher.send(help_msg)

    async def card(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        try:
            if not ess.check(event, 'Card'):
                return None
            args = arg.extract_plain_text().split(' ')

            if args == [''] or args is None or args[0] == 'help':
                await self.help(matcher)
                return None
            
            # input
            name = args[0]
            target_group = None
            target_table = None
            # output
            now_id = ''
            now_table = ''
            now_score = 0

            # input parse
            for arg in args:
                if arg in groups.keys():
                    target_group = groups[arg]
                elif arg in groups.values():
                    target_group = arg
                elif arg in tables:
                    target_table = arg
            
            for regex, key in self.regex.items():
               if re.match(regex, name, re.I):
                   name = key        
            
            for table in tables:
                if target_table and target_table != table:
                    continue
                table_attr: BaseClass = getattr(self, table)
                for tuple in table_attr:
                    if target_group:
                        if tuple[3] != target_group:
                            continue
                        score = max(fuzz.ratio(B2Q(name), tuple[1]), fuzz.ratio(name, tuple[2]))
                        if score > now_score:
                            now_id = tuple[0]
                            now_table = table
                            now_score = score
                    else:
                        score = max(fuzz.ratio(B2Q(name), tuple[1]), fuzz.ratio(name, tuple[2])) + (1 if tuple[3] == 'ordinary' else 0)
                        if score > now_score:
                            now_id = tuple[0]
                            now_table = table
                            now_score = score
            
            if now_score < plugin_config.match_score:
                await matcher.send('没有这张卡啦~')
                return None

            if now_table.startswith('unit'):
                img = f'http://interface.100oj.com/interface/render/cardunit.php?key={now_id}'
                await matcher.send(MessageSegment.image(img))
                return None
            else:
                img = f'http://interface.100oj.com/interface/render/{now_table.lower()}.php?key={now_id}'
                await matcher.send(MessageSegment.image(img))
                return None                               

        except Exception as e:
            await matcher.send('诶鸭出错啦~')
            raise e

    async def icon(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        try:
            if not ess.check(event, 'Card'):
                return None
            args = arg.extract_plain_text().split(' ')

            if args == [''] or args is None or args[0] == 'help':
                await self.help(matcher)
                return None


            name = args[0]
            target_group = None
            target_table = None

            for arg in args:
                if arg in groups.keys():
                    target_group = groups[arg]
                elif arg in groups.values():
                    target_group = arg
                elif arg in tables:
                    target_table = arg   

            id = ''
            now_score = 0
            for table in tables:
                if target_table and target_table != table:
                    continue
                table_attr: BaseClass = getattr(self, table)
                for tuple in table_attr:
                    if target_group:
                        if tuple[3] != target_group:
                            continue
                        score = max(fuzz.ratio(B2Q(name), tuple[1]), fuzz.ratio(name, tuple[2]))
                        if score > now_score:
                            id = tuple[0]
                            now_score = score
                    else:
                        score = max(fuzz.ratio(B2Q(name), tuple[1]), fuzz.ratio(name, tuple[2])) + (1 if tuple[3] == 'ordinary' else 0)
                        if score > now_score:
                            id = tuple[0]
                            now_score = score
            
            if now_score < plugin_config.match_score:
                await matcher.send('没有这张卡啦~')
                return None

            img = f'http://interface.100oj.com/interface/util/icon.php?key={id}&size=256&lossless=true'
            await matcher.send(MessageSegment.image(img))
            return None                     
        except Exception as e:
            await matcher.send('诶鸭出错啦~')
            raise e

card: Card = Card()
driver = get_driver()

@driver.on_startup
async def init_card() -> None:
    await card.get_data()


import os
from pathlib import Path
from typing import Union

from nonebot.adapters.onebot.v11 import MessageSegment, GroupMessageEvent, PrivateMessageEvent
from nonebot.params import RegexStr
from nonebot.matcher import Matcher

from .Ess import ess

class Emote: 
    def __init__(self) -> None:
        pass

    async def emote(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], key: str = RegexStr()) -> None:
        if not ess.check(event, 'Emote'):
            return None

        key = key.replace(':', '').lower()
        folder_path = os.path.join(os.path.dirname(__file__), 'resources', 'emotes')
        img_name = f'{key}.png'
        img = Path(folder_path) / img_name
        poppo = Path(folder_path) / 'poppo.png'
        try:
            await matcher.send(MessageSegment.image(img))
        except:
            await matcher.send(MessageSegment.image(poppo))

emote = Emote()


import datetime
import json
import os
import re
from typing import Any, Dict, List, Union

from nonebot import logger
from nonebot.matcher import Matcher
from nonebot.params import CommandArg
from nonebot.adapters.onebot.v11 import Bot, Message, GroupMessageEvent, PrivateMessageEvent

from .Config import plugin_config

ess_file_path: str = os.path.join(plugin_config.oj_data_path, 'essentialx.json')
init_json: Dict[str, Union[str, List[str], Dict[int, Dict[str, Union[int, str, bool, List[str], Dict[str, str]]]]]]

init_json = {
    "version": 3,
    "current_modules": ["Card", "Deck", "Emote", "Le", "Mixer", "Stats"],
    "group_config_list": {
        "114514": {
            "module_disabled": [
                "Le"
            ],
            "scheduler": {
                "time": "0 0 * * *",
                "content": "欢迎新人！"     
            },
            "greet": "欢迎新人！",
            "isGreet": False,
            "le_cd": 60,
            "le_max": 10
        }
    },
    "blacklist": []
}

help_msg: str = '''Ess模块，需要是超级管理员/群管理/群主
    #ess module enable <modulename> [groupid]
    #ess module disable <modulename> [groupid]
    #ess module list [groupid]
    #ess mute <uid> [time] [reason]'''

class Args:
    def __init__(self, bot: Bot, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message) -> None:
        self.bot: Bot = bot
        self.event: Union[GroupMessageEvent, PrivateMessageEvent] = event

        self.invaild: bool = False
        self.handle: str = None
        self.operate: str = None
        self.target_module: str = None
        self.target_uid: int = None
        self.target_gid: int = None
        self.send_uid: int = event.user_id
        self.send_gid: int = event.group_id if isinstance(event, GroupMessageEvent) else None
        self.is_admin: bool = event.sender.role in ["admin", "owner"] if isinstance(event, GroupMessageEvent) else False
        self.is_superuser: bool = str(event.user_id) in bot.config.superusers

        self.extra_data: Dict[str, Any] = {}


        list_args = arg.to_rich_text().split(' ')

        while len(list_args) < 5:
            list_args.append('')
        logger.debug(f'参数解析结果：{list_args}')

        for i in range(0,5):
            list_args[i] = re.sub(r'\[at:qq=(.*?)\]', r'\1', list_args[i])

        match list_args[0]:
            case 'module' | '-m':
                self.handle = 'module'

                match list_args[1]:
                    case 'list' | '-l' | 'ls':
                        self.operate = 'list'

                        self.invaild = True # #ess module list [gid]
                        logger.trace('ess module list [gid] trigger.')
                        self.target_gid = int(list_args[2]) if list_args[2].isdigit() else self.send_gid

                    case 'enable' | '-on':
                        self.operate = 'enable'
                        if list_args[2] in ess.config['current_modules']:
                            self.invaild = True # #ess module enable <modulename> [gid]
                            logger.trace(f'ess module enable <modulename> [gid] trigger.')
                            self.target_module = list_args[2]
                            self.target_gid = int(list_args[3]) if list_args[3].isdigit() else self.send_gid


                    case 'disable' | '-off':
                        self.operate = 'disable'
                        if list_args[2] in ess.config['current_modules']:
                            self.invaild = True # #ess module disable <modulename> [gid]
                            logger.trace(f'ess module disable <modulename> [gid] trigger.')
                            self.target_module = list_args[2]
                            self.target_gid = int(list_args[3]) if list_args[3].isdigit() else self.send_gid
            
            case 'mute':
                self.handle = 'mute'
                if list_args[1].isdigit():
                    self.invaild = True # #ess mute <uid> [time] [reason]
                    logger.trace(f'ess mute <id> [time] [reason] trigger.')
                    self.target_uid = int(list_args[1])
                    self.extra_data['mute_reason'] = list_args[3]
                    if list_args[2] != '':
                        if list_args[2].endswith('m'):
                            self.extra_data['mute_time'] = int(list_args[2][:-1]) * 60
                        elif list_args[2].endswith('h'):
                            self.extra_data['mute_time'] = int(list_args[2][:-1]) * 3600
                        elif list_args[2].endswith('d'):
                            self.extra_data['mute_time'] = int(list_args[2][:-1]) * 3600 * 24
                        elif list_args[2].endswith('s'):
                            self.extra_data['mute_time'] = int(list_args[2][:-1])
                        elif list_args[2].isdigit():
                            self.extra_data['mute_time'] = int(list_args[2])
                        else:
                            self.invaild = False
                    else:
                        self.extra_data['mute_time'] = 30

            case 'save' | '-s':
                self.invaild = True # #ess save
                logger.trace(f'ess save trigger.')
                self.handle = 'save'
            case 'load' | '-l':
                self.invaild = True # #ess load
                logger.trace(f'ess load trigger.')
                self.handle = 'load'
                
            case _:
                pass



class Ess:
    def __init__(self) -> None:
        if not os.path.exists(ess_file_path):
            if not os.path.exists(plugin_config.oj_data_path):
                os.makedirs(plugin_config.oj_data_path)
            self.init_json()  

        self.load_json()

        if self.config["version"] < 3:
            new_config = init_json
            for module in self.config["modules"]:
                for gid in self.config["modules"][module]:
                    if str(gid) not in new_config['group_config_list']:
                        new_config['group_config_list'][str(gid)] = {
                            "module_disabled": [],
                            "scheduler": {},
                            "greet": "欢迎新人！",
                            "isGreet": False,
                            "le_cd": 60,
                            "le_max": 10
                        }
                    new_config['group_config_list'][gid]['module_disabled'].append(module)
            self.config = new_config
            self.save_json()           


    def init_json(self):
        with open(ess_file_path, 'w+') as f:
            f.write(json.dumps(init_json, indent=4))

    def load_json(self):
        with open(ess_file_path, 'r', encoding='utf-8') as f:
            data: Dict[str, Union[str, List[str], Dict[int, Dict[str, Union[int, str, bool, List[str], Dict[str, str]]]]]]
            data = json.load(f)
            self.config = data

    def save_json(self):
        with open(ess_file_path, 'w', encoding='utf-8') as f:
            json_data = json.dumps(self.config, indent=4)
            f.write(json_data)

    def check(self, event: Union[GroupMessageEvent, PrivateMessageEvent], module: str = None) -> bool:
        # Todo: Group Config
        if event.user_id in self.config['blacklist']:
            return False
        
        if isinstance(event, PrivateMessageEvent):
            return True
        
        gid = event.group_id if isinstance(event, GroupMessageEvent) else None
        
        self.check_gid(gid)
            
        if module and module in self.config['group_config_list'][gid]['module_disabled']:
            return False
              
        return True

    def check_gid(self, gid: Union[int, str]) -> None:
        if str(gid) not in self.config['group_config_list']:
            self.config['group_config_list'][str(gid)] = {
                "module_disabled": [],
                "scheduler": {},
                "greet": "欢迎新人！",
                "isGreet": False,
                "le_cd": 60,
                "le_max": 10
            }
        self.save_json()

  
    async def save(self, matcher: Matcher, args: Args):
        if not args.is_superuser:
            await matcher.finish('诶鸭你没有权限这样做啦！')
        
        self.save_json()
        await matcher.finish('已保存配置~')

    async def load(self, matcher: Matcher, args: Args):
        if not args.is_superuser:
            await matcher.finish('诶鸭你没有权限这样做啦！')
        
        self.load_json()
        await matcher.finish('已载入配置~')
    

    async def help(self, matcher: Matcher) -> None:
        await matcher.finish(help_msg)

    async def module(self, matcher: Matcher, args: Args) -> None:

        if args.target_gid == None:
            await matcher.finish('呜，请指定群号！')

        if args.target_gid != args.send_gid and not args.is_superuser:
            await matcher.finish('诶鸭你没有权限这样做啦！')
            
        self.check_gid(args.target_gid)
        
        if args.operate == 'enable':
            if args.target_module in self.config['group_config_list'][str(args.target_gid)]['module_disabled']:
                self.config['group_config_list'][str(args.target_gid)]['module_disabled'].remove(args.target_module)
            self.save_json()
            await matcher.finish(f'已在{args.target_gid}成功启用{args.target_module}模块！')

        elif args.operate == 'disable':
            if args.target_module not in self.config['group_config_list'][str(args.target_gid)]['module_disabled']:
                self.config['group_config_list'][str(args.target_gid)]['module_disabled'].append(args.target_module)
            self.save_json()
            await matcher.finish(f'已在{args.target_gid}成功禁用{args.target_module}模块！')

        elif args.operate == 'list':
            enable_modules = []
            avaliable_modules = []
            
            for module in self.config['current_modules']:
                enable_modules.append(module) if module not in self.config['group_config_list'][str(args.target_gid)]['module_disabled'] else avaliable_modules.append(module)
                       
            msg1 = "\n- " + "\n- ".join(enable_modules) if enable_modules != [] else ""
            msg2 = "\n- " + "\n- ".join(avaliable_modules) if avaliable_modules != [] else ""
            await matcher.finish(f'群{args.target_gid}\n启用的模块: {msg1}\n可用模块: {msg2}')

        else:
            await self.help(matcher)
    async def mute(self, matcher: Matcher, args: Args):
        ban_time = args.extra_data['mute_time']

        now_time = datetime.datetime.now()
        unban_time = now_time + datetime.timedelta(seconds=ban_time)
        unban_time_str = unban_time.strftime("%Y年%m月%d日%H时%M分%S秒")
        try:
            info = await args.bot.get_group_member_info(group_id=args.send_gid, user_id=args.target_uid)
            name = info['card'] if info['card'] is not None and info['card'] != '' else info['nickname']
            await args.bot.set_group_ban(group_id=args.send_gid, user_id=args.target_uid, duration=ban_time)
        except Exception as e:
            await matcher.send('禁言失败(')

        await matcher.send(f'被禁者:{name}\n禁言原因: {args.extra_data["mute_reason"]}\n被禁时间: {str(ban_time)}秒\n解禁时间: {unban_time_str}')

    async def ess(self, bot: Bot, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        args = Args(bot, event, arg)

        if not args.is_admin and not args.is_superuser:
            return None
        
        if not args.invaild:
            await self.help(matcher)

        if hasattr(args, "handle"):
            await getattr(self, args.handle)(matcher, args)
            
        
ess = Ess()



from typing import Union

from nonebot.adapters.onebot.v11 import Message, MessageSegment, GroupMessageEvent, PrivateMessageEvent
from nonebot.matcher import Matcher
from nonebot.params import CommandArg

from .Ess import ess

class Deck:
    def __init__(self) -> None:
        pass

    async def deck(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        if not ess.check(event, 'Deck'):
            return None

        deck = arg.extract_plain_text()

        if deck == '' or deck is None or len(deck) != 12:
            await matcher.finish('请输入正确的卡组编码~')

        img = f'https://interface.100oj.com/deck/render.php?deck={deck}'
        await matcher.send(MessageSegment.image(img))

deck = Deck()


import datetime
import json
import time
from typing import Union

import aiohttp
from cachetools import TTLCache

from nonebot.adapters.onebot.v11 import GroupMessageEvent, PrivateMessageEvent, Message
from nonebot.matcher import Matcher
from nonebot.params import CommandArg

from .Ess import ess

mixers_list = ["空袭 +10%","增幅 +10%","返回 +10%","炸弹 +10%","恩惠 +0%","混乱 +25%","冰冻 +10%","布雷 +25%","奇迹 +0%","随机传送 +25%","恢复 +0%","疾走 +10%","宝箱 +0%","BOSS HP+10 +25%","追星者 +0%","乞丐 +25%","玩家ATK+1 +0%","玩家DEF+1 +0%","玩家EVD+1 +0%","爆燃 +10%","翻转 +10%","BOSS ATK+1 +25%","BOSS DEF+1 +25%","BOSS EVD+1 +25%","鬼牌 +0%","破产 +25%","粘液 +25%","孢子 +10%",""]

class Mixer:
    def __init__(self):
        self.cache = TTLCache(maxsize=2, ttl=43200)

    def get_today_time(self) -> int:
        return int(time.mktime(datetime.date.today().timetuple())) + 57600
    
    async def get_data(self) -> dict:
        cache_key = "content_min_js"
        if cache_key in self.cache:
            return self.cache[cache_key]
        
        async with aiohttp.ClientSession() as session:
            req = await session.get('https://interface.100oj.com/mixer/js/content.min.js')
            if req.status == 200:
                json_data = await req.text()
                data = json.loads(json_data.replace('var ms = ', ''))
                self.cache[cache_key] = data
                return data
            else:
                return None

    async def mixer(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        if not ess.check(event, 'Mixer'):
            return None
        
        self.data = await self.get_data()

        if not self.data:
            await matcher.finish('诶鸭出错啦~')
            
        text: str = arg.extract_plain_text().replace(' ', '')
        day = min(int(text), 850) if (text := text[:3]).isdigit() else 2

        if day == 1:
            date = '昨日混合器：'
        elif day == 2:
            date = '今日混合器：'
        elif day == 0:
            date = f'2日前混合器：' 
        else:
            date = f'{day - 2}日后混合器：'
            

        query_time = self.get_today_time() + (day - 1) * 86400
        key = str(hex(query_time))

        today_mixer = self.data.get(key, None)
        if today_mixer:
            if len(today_mixer) == 3:
                await matcher.finish(date + f'\n「{mixers_list[today_mixer[0]]}」, 「{mixers_list[today_mixer[1]]}」, 「{mixers_list[today_mixer[2]]}」')
            else:
                await matcher.finish(date + f'\n「{mixers_list[today_mixer[0]]}」, 「{mixers_list[today_mixer[1]]}」')
        else:
            await matcher.finish('数据库里还没有这一天的混合器数据哦~')

mixer: Mixer = Mixer()
